using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml;
using System.ServiceModel;
using System.Runtime.Serialization;

namespace ClockMark
{
    public enum Occurence : ushort
    {
        Once = 0,
        Minutely,
        Hourly,
        Daily,
        Weekly,
        Monthly,
        Yearly
    }

    /// <summary>
    /// base Alarm type saved in the xml file
    /// </summary>
    public class Alarm
    {
        #region Variables

        private string name; // (ObligatorY) title of the alarm
        private string description; // (optional) description
        private Occurence occurence; // (ObligatorY) occurence type as defined by its enum type
        private DateTime occurenceTime; // (ObligatorY) occurence time
        private DateTime occurenceDate; // (optional) occurence date
        private ushort occurenceMinutes; // (optional) occurence minutes (1-59)
        private ushort[] occurenceDays; // (optional) occurence days list (1-7 : Mon-Sun)
        private bool playSound; // (optional)
        private string playSoundFile; // (optional)
        private bool executeApplication; // (optional)
        private string executeApplicationFile; // (optional)
        private bool shutdownPC; // (optional)

        #endregion

        #region Properties

        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        public string Description
        {
            get { return description; }
            set { description = value; }
        }

        public Occurence Occurence
        {
            get { return occurence; }
            set { occurence = value; }
        }

        public DateTime OccurenceTime
        {
            get { return occurenceTime; }
            set { occurenceTime = value; }
        }

        public DateTime OccurenceDate
        {
            get { return occurenceDate; }
            set { occurenceDate = value; }
        }

        public ushort OccurenceMinutes
        {
            get { return occurenceMinutes; }
            set { occurenceMinutes = value; }
        }

        public ushort[] OccurenceDays
        {
            get { return occurenceDays; }
            set { occurenceDays = value; }
        }

        public bool PlaySound
        {
            get { return playSound; }
            set { playSound = value; }
        }

        public string PlaySoundFile
        {
            get { return playSoundFile; }
            set { playSoundFile = value; }
        }

        public bool ExecuteApplication
        {
            get { return executeApplication; }
            set { executeApplication = value; }
        }

        public string ExecuteApplicationFile
        {
            get { return executeApplicationFile; }
            set { executeApplicationFile = value; }
        }

        public bool ShutdownPC
        {
            get { return shutdownPC; }
            set { shutdownPC = value; }
        }

        #endregion

        #region Constructors

        public Alarm()
        {
            this.Name = "New Alarm";
            this.Description = String.Empty;
            this.Occurence = Occurence.Daily;
            this.OccurenceTime = DateTime.Now;
            this.OccurenceDate = DateTime.Today;
            this.OccurenceMinutes = 15;
            this.OccurenceDays = new ushort[7] { 1, 1, 1, 1, 1, 0, 0 };
            this.PlaySound = false;
            this.PlaySoundFile = String.Empty;
            this.ExecuteApplication = false;
            this.ExecuteApplicationFile = String.Empty;
            this.ShutdownPC = false;
        }

        public Alarm(string Name, string Description,
            Occurence Occurence,
            DateTime OccurenceTime, DateTime OccurenceDate, 
            ushort OccurenceMinutes, ushort[] OccurenceDays, 
            bool PlaySound, string PlaySoundFile,
            bool ExecuteApplication, string ExecuteApplicationFile,
            bool ShutdownPC)
        {
            this.Name = Name;
            this.Description = Description;
            this.Occurence = Occurence;
            this.OccurenceTime = OccurenceTime;
            this.OccurenceDate = OccurenceDate;
            this.OccurenceMinutes = OccurenceMinutes;
            this.OccurenceDays = OccurenceDays;
            this.PlaySound = PlaySound;
            this.PlaySoundFile = PlaySoundFile;
            this.ExecuteApplication = ExecuteApplication;
            this.ExecuteApplicationFile = ExecuteApplicationFile;
            this.ShutdownPC = ShutdownPC;
        }

        #endregion
    }

    /// <summary>
    /// Alarms is a class that gives utils for accessing the user xml file
    /// </summary>
    public class Alarms
    {
        private string strPath;
        private string strFileName;
        private string strUser;
        private ArrayList alAlarms;

        public ArrayList AlAlarms
        {
            get { return alAlarms; }
            set { alAlarms = value; }
        }

        public Alarms(string[] uuid)
        {
            if (uuid[0] == null)
                return;

            this.strPath = Directory.GetCurrentDirectory();
            this.strUser = uuid[0];
            this.strFileName = this.strPath +
                Path.DirectorySeparatorChar +
                this.strUser + "Alarms.xml";

            this.AlAlarms = this.LoadAlarms();
            if (this.AlAlarms == null)
                this.CreateAlarmsList();
        }

        public Alarms(string path, string[] uuid)
        {
            if (uuid[0] == null)
                return;

            this.strPath = path;
            this.strUser = uuid[0];
            this.strFileName = this.strPath +
                Path.DirectorySeparatorChar +
                this.strUser + "Alarms";

            this.AlAlarms = this.LoadAlarms();
            if (this.AlAlarms == null)
                this.CreateAlarmsList();
        }

        /// <summary>
        /// Check if Alarm with the same name is available
        /// </summary>
        /// <param name="name">Alarm Name</param>
        /// <returns></returns>
        public bool IsAlarm(string name)
        {
            if (this.AlAlarms == null)
                return false;

            foreach (Alarm alarm in this.AlAlarms)
            {
                if (alarm.Name == name)
                    return true;
            }

            return false;
        }

        /// <summary>
        /// Add new alarm
        /// </summary>
        /// <param name="alarm">New Alarm</param>
        /// <returns></returns>
        public bool AddAlarm(Alarm alarm)
        {
            if (this.AlAlarms == null)
                return false;

            if (this.IsAlarm(alarm.Name))
                return false;

            this.AlAlarms.Add(alarm);

            return true;
        }

        /// <summary>
        /// Delete the existing alarm
        /// </summary>
        /// <param name="name">Existing Alarm</param>
        /// <returns></returns>
        public bool DeleteAlarm(string name)
        {
            if (this.AlAlarms == null)
                return false;

            foreach (Alarm alarm in this.AlAlarms)
            {
                if (alarm.Name == name)
                {
                    this.AlAlarms.Remove(alarm);
                    return true;
                }
            }

            return false;
        }

        /// <summary>
        /// Create empty alarms file for user
        /// </summary>
        /// <returns></returns>
        private bool CreateAlarmsList()
        {
            try
            {
                XmlDocument xd = new XmlDocument();

                XmlDeclaration xdec = xd.CreateXmlDeclaration("1.0", "utf-8", null);
                xd.AppendChild(xdec);

                XmlElement xroot = xd.CreateElement("Alarms");
                xd.AppendChild(xroot);

                xd.Save(this.strFileName);
            }
            catch (Exception ex)
            {
                System.Diagnostics.Trace.WriteLine("Could not create empty alarm list.");
                System.Diagnostics.Trace.WriteLine(ex.Message);
                return false;
            }

            return true;
        }

        /// <summary>
        /// Get the alarms list from the specific user alarms xml file
        /// </summary>
        /// <returns></returns>
        private ArrayList LoadAlarms()
        {
            this.AlAlarms = new ArrayList();

            try
            {
                if (File.Exists(this.strFileName))
                {
                    XmlDocument xd = new XmlDocument();
                    xd.Load(this.strFileName);

                    foreach (XmlNode xn in xd.SelectNodes("Alarms/Alarm"))
                    {
                        Alarm alarm = new Alarm();
                        
                        alarm.Name = xn.SelectSingleNode("Name").InnerText;
                        alarm.Description = xn.SelectSingleNode("Description").InnerText;

                        switch (xn.SelectSingleNode("Occurence").InnerText)
                        {
                            case "Once":
                                alarm.Occurence = Occurence.Once;
                                break;
                            case "Minutely":
                                alarm.Occurence = Occurence.Minutely;
                                break;
                            case "Hourly":
                                alarm.Occurence = Occurence.Hourly;
                                break;
                            case "Daily":
                                alarm.Occurence = Occurence.Daily;
                                break;
                            case "Weekly":
                                alarm.Occurence = Occurence.Weekly;
                                break;
                            case "Monthly":
                                alarm.Occurence = Occurence.Monthly;
                                break;
                            case "Yearly":
                                alarm.Occurence = Occurence.Yearly;
                                break;
                            default:
                                alarm.Occurence = Occurence.Once;
                                break;
                        }

                        alarm.OccurenceDate = DateTime.Parse(xn.SelectSingleNode("OccurenceDate").InnerText);
                        alarm.OccurenceTime = DateTime.Parse(xn.SelectSingleNode("OccurenceTime").InnerText);
                        alarm.OccurenceMinutes = ushort.Parse(xn.SelectSingleNode("OccurenceMinutes").InnerText);
                        
                        string days = xn.SelectSingleNode("OccurenceDays").InnerText;
                        for (int i = 0; i < 7; i++)
                            alarm.OccurenceDays[i] = Convert.ToUInt16(days[i]);

                        alarm.PlaySound = Boolean.Parse(xn.SelectSingleNode("PlaySound").InnerText);
                        alarm.PlaySoundFile = xn.SelectSingleNode("PlaySoundFile").InnerText;
                        alarm.ExecuteApplication = Boolean.Parse(xn.SelectSingleNode("ExecuteApplication").InnerText);
                        alarm.ExecuteApplicationFile = xn.SelectSingleNode("ExecuteApplicationFile").InnerText;
                        alarm.ShutdownPC = Boolean.Parse(xn.SelectSingleNode("ShutdownPC").InnerText);

                        this.AlAlarms.Add(alarm);
                    }
                }
                else
                {
                    System.Diagnostics.Trace.WriteLine("Alarm file does not exist");
                    return null;
                }
            }
            catch (Exception ex)
            {
                System.Diagnostics.Trace.WriteLine("Could not load alarms.");
                System.Diagnostics.Trace.WriteLine(ex.Message);
                return null;
            }

            return this.AlAlarms;
        }

        /// <summary>
        /// Save the alarms list into the specific user alarms xml file
        /// </summary>
        /// <returns></returns>
        public bool SaveAlarms()
        {
            if (this.AlAlarms == null)
                return false;

            try
            {
                XmlDocument xd = new XmlDocument();

                XmlDeclaration xdec = xd.CreateXmlDeclaration("1.0", "utf-8", null);
                xd.AppendChild(xdec);

                XmlElement xroot = xd.CreateElement("Alarms");
                xd.AppendChild(xroot);

                foreach (Alarm alarm in this.AlAlarms)
                {
                    XmlElement xea = xd.CreateElement("Alarm");
                    xroot.AppendChild(xea);

                    XmlElement xe = xd.CreateElement("Name");
                    XmlText xt = xd.CreateTextNode(alarm.Name);
                    xe.AppendChild(xt);
                    xea.AppendChild(xe);

                    xe = xd.CreateElement("Description");
                    xt = xd.CreateTextNode(alarm.Description);
                    xe.AppendChild(xt);
                    xea.AppendChild(xe);

                    xe = xd.CreateElement("Occurence");
                    xt = xd.CreateTextNode(alarm.Occurence.ToString());
                    xe.AppendChild(xt);
                    xea.AppendChild(xe);

                    xe = xd.CreateElement("OccurenceTime");
                    xt = xd.CreateTextNode(alarm.OccurenceTime.ToString());
                    xe.AppendChild(xt);
                    xea.AppendChild(xe);

                    xe = xd.CreateElement("OccurenceDate");
                    xt = xd.CreateTextNode(alarm.OccurenceDate.ToString());
                    xe.AppendChild(xt);
                    xea.AppendChild(xe);

                    xe = xd.CreateElement("OccurenceMinutes");
                    xt = xd.CreateTextNode(alarm.OccurenceMinutes.ToString());
                    xe.AppendChild(xt);
                    xea.AppendChild(xe);

                    char[] days = new char[7] { '0', '0', '0', '0', '0', '0', '0' };
                    for (int i = 0; i < 7; i++)
                        days[i] = Convert.ToChar(alarm.OccurenceDays[i]);
                    string strDays = new string(days);

                    xe = xd.CreateElement("OccurenceDays");
                    xt = xd.CreateTextNode(strDays);
                    xe.AppendChild(xt);
                    xea.AppendChild(xe);

                    xe = xd.CreateElement("PlaySound");
                    xt = xd.CreateTextNode(alarm.PlaySound.ToString());
                    xe.AppendChild(xt);
                    xea.AppendChild(xe);

                    xe = xd.CreateElement("PlaySound");
                    xt = xd.CreateTextNode(alarm.PlaySound.ToString());
                    xe.AppendChild(xt);
                    xea.AppendChild(xe);

                    xe = xd.CreateElement("PlaySoundFile");
                    xt = xd.CreateTextNode(alarm.PlaySoundFile.ToString());
                    xe.AppendChild(xt);
                    xea.AppendChild(xe);

                    xe = xd.CreateElement("ExecuteApplication");
                    xt = xd.CreateTextNode(alarm.ExecuteApplication.ToString());
                    xe.AppendChild(xt);
                    xea.AppendChild(xe);

                    xe = xd.CreateElement("ExecuteApplicationFile");
                    xt = xd.CreateTextNode(alarm.ExecuteApplicationFile.ToString());
                    xe.AppendChild(xt);
                    xea.AppendChild(xe);

                    xe = xd.CreateElement("ShutdownPC");
                    xt = xd.CreateTextNode(alarm.ShutdownPC.ToString());
                    xe.AppendChild(xt);
                    xea.AppendChild(xe);
                }

                xd.Save(this.strFileName);
            }
            catch (Exception ex)
            {
                System.Diagnostics.Trace.WriteLine("Could not save alarms.");
                System.Diagnostics.Trace.WriteLine(ex.Message);
                return false;
            }

            return true;
        }

        public bool FromString(string alarms)
        {
            try
            {
                XmlDocument xd = new XmlDocument();
                xd.InnerXml = alarms;

                xd.Save(this.strFileName);
                return true;
            }
            catch (Exception ex)
            {
                System.Diagnostics.Trace.WriteLine("Could not convert xml string to settings.");
                System.Diagnostics.Trace.WriteLine(ex.Message);
                return false;
            }
        }

        public override string ToString()
        {
            try
            {
                XmlDocument xd = new XmlDocument();

                if (File.Exists(this.strFileName))
                {
                    xd.Load(this.strFileName);
                    return xd.InnerXml.ToString();
                }
                else
                {
                    System.Diagnostics.Trace.WriteLine("Alarm file does not exist");
                    return null;
                }
            }
            catch (Exception ex)
            {
                System.Diagnostics.Trace.WriteLine("Could not convert alarms to xml string.");
                System.Diagnostics.Trace.WriteLine(ex.Message);
                return null;
            }
        }
    }
}

