using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.Text;
using System.Xml;
using System.IO;

namespace ClockMarkServer
{
    public class ClockMarkServiceType : IClockMarkService
    {
        internal Users users;

        public ClockMarkServiceType()
        {
            // in its constructor the Users class loads existing users from xml file
            this.users = new Users();

            System.Diagnostics.Trace.Listeners.Clear();
            System.Diagnostics.Trace.Listeners.Add(new System.Diagnostics.ConsoleTraceListener());
            System.Diagnostics.Trace.WriteLine("Constructing service.");
        
        }

        public void Dispose()
        {
            System.Diagnostics.Trace.WriteLine("Constructing service.");
            Dispose();
            // This object will be cleaned up by the Dispose method.
            // Therefore, you should call GC.SupressFinalize to
            // take this object off the finalization queue
            // and prevent finalization code for this object
            // from executing a second time.
            GC.SuppressFinalize(this);
        }

        #region Service

        public bool IsServiceUp()
        {
            System.Diagnostics.Trace.WriteLine("Service is up.");
            return true;
        }

        #endregion

        #region User Handling

        public bool IsExistingUser(string[] uuid)
        {
            if ((uuid == null) ||
                (uuid.Length != 4) ||
                (uuid.LongLength == 0) ||
                (this.users == null))
            {
                return false;
            }

            if (this.users.IsUser(uuid[0]))
            {
                System.Diagnostics.Trace.WriteLine("User [" + uuid[0] + "] exist.");
                return true;
            }

            return false;
        }

        public bool IsUserLoginValid(string[] uuid)
        {
            if ((uuid == null) ||
                (uuid.Length != 4) ||
                (uuid.LongLength == 0) ||
                (this.users == null))
            {
                return false;
            }

            if (this.users.IsLoginValid(uuid[0], uuid[1]))
            {
                System.Diagnostics.Trace.WriteLine("User [" + uuid[0] + "] has valid login.");
                return true;
            }

            return false;
        }

        public bool CreateNewUser(string[] uuid)
        {
            if ((uuid == null) ||
                (uuid.Length != 4) ||
                (uuid.LongLength == 0) ||
                (this.users == null))
            {
                return false;
            }

            if (!this.users.IsUser(uuid[0]) &&
                (this.users.AddUser(uuid[0], uuid[1])))
            {
                System.Diagnostics.Trace.WriteLine("User [" + uuid[0] + "] has been created.");
                return true;
            }

            return false;
        }

        public bool DeleteExistingUser(string[] uuid)
        {
            if ((uuid == null) ||
                (uuid.Length != 4) ||
                (uuid.LongLength == 0) ||
                (this.users == null))
            {
                return false;
            }

            if (this.users.IsUser(uuid[0]) &&
                this.users.IsLoginValid(uuid[0], uuid[1]) &&
                this.users.DeleteUser(uuid[0]))
            {
                System.Diagnostics.Trace.WriteLine("User [" + uuid[0] + "] has been deleted.");
                return true;
            }

            return false;
        }

        #endregion

        #region XML file handling

        public DateTime GetXmlFileTimeStamp(string[] uuid, string xmlFileName)
        {
            if ((uuid == null) ||
                (uuid.Length != 4) ||
                (uuid.LongLength == 0) ||
                (this.users == null))
                return DateTime.MinValue;

            if (!this.users.IsUser(uuid[0]) || !this.users.IsLoginValid(uuid[0], uuid[1]))
                return DateTime.MinValue;

            if (xmlFileName == null)
                return DateTime.MinValue;

            try
            {
                string xf = Directory.GetCurrentDirectory() +
                    Path.DirectorySeparatorChar +
                    uuid[0] + "_" +
                    xmlFileName;

                if (!File.Exists(xf))
                {
                    System.Diagnostics.Trace.WriteLine("File [" + xf + "] could not be found!");
                    return DateTime.MinValue;
                }

                System.Diagnostics.Trace.WriteLine("File [" + xf + "] has timestamp [" + File.GetLastAccessTime(xf) + "].");
                return File.GetLastAccessTime(xf);
            }
            catch (Exception ex)
            {
                System.Diagnostics.Trace.WriteLine("File [" + xmlFileName + "] could not be found!\r\n" + ex.Message);
                return DateTime.MinValue;
            }
        }

        public string GetXmlFile(string[] uuid, string xmlFileName)
        {
            if ((uuid == null) ||
                (uuid.Length != 4) ||
                (uuid.LongLength == 0) ||
                (this.users == null))
                return null;

            if (!this.users.IsUser(uuid[0]) || !this.users.IsLoginValid(uuid[0], uuid[1]))
                return null;

            if (xmlFileName == null)
                return null;

            try
            {
                XmlDocument xd = new XmlDocument();
                string xf = Directory.GetCurrentDirectory() +
                    Path.DirectorySeparatorChar +
                    uuid[0] + "_" +
                    xmlFileName;

                if (File.Exists(xf))
                {
                    xd.Load(xf);
                    System.Diagnostics.Trace.WriteLine("Load file [" + xf + "].");
                }
                else
                {
                    System.Diagnostics.Trace.WriteLine("File [" + xf + "] could not be loaded!");
                    return null;
                }

                return String.Copy(xd.InnerXml);
            }
            catch (Exception ex)
            {
                System.Diagnostics.Trace.WriteLine("File [" + xmlFileName + "] could not be loaded!\r\n" + ex.Message);
                return null;
            }
        }

        public bool PutXmlFile(string[] uuid, string xmlFileName, string xmlFile)
        {
            if ((uuid == null) ||
                (uuid.Length != 4) ||
                (uuid.LongLength == 0) ||
                (this.users == null))
                return false;

            if (!this.users.IsUser(uuid[0]) || !this.users.IsLoginValid(uuid[0], uuid[1]))
                return false;

            if ((xmlFileName == null) || (xmlFile == null))
                return false;

            try
            {
                XmlDocument xd = new XmlDocument();
                xd.InnerXml = String.Copy(xmlFile);
                string xf = Directory.GetCurrentDirectory() +
                    Path.DirectorySeparatorChar +
                    uuid[0] + "_" +
                    xmlFileName;

                if (File.Exists(xf))
                {
                    string xfbackup = Path.GetDirectoryName(xf) +
                        Path.DirectorySeparatorChar +
                        Path.GetFileNameWithoutExtension(xf) +
                        "." + DateTime.Now.ToFileTime().ToString() +
                        Path.GetExtension(xf);

                    if (File.Exists(xfbackup))
                        File.Delete(xfbackup);

                    File.Copy(xf, xfbackup);
                    System.Diagnostics.Trace.WriteLine("Backup file [" + xf + "] to [" + xfbackup + "].");
                }

                xd.Save(xf);
                System.Diagnostics.Trace.WriteLine("Save file [" + xf + "].");
             
                return true;
            }
            catch (Exception ex)
            {
                System.Diagnostics.Trace.WriteLine("File [" + xmlFileName + "] could not be saved!\r\n" + ex.Message);
                return false;
            }
        }

        #endregion
    }
}
