using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Reflection;
using System.Xml;
using System.Runtime.Serialization;

namespace ClockMarkServer
{
    /// <summary>
    /// base user type saved in the xml file
    /// Server only type!
    /// </summary>
    public class User
    {
        public string UserName;
        public string Password;
        public DateTime LastLogin;

        public User()
        {
            this.LastLogin = DateTime.Now;
        }

        public User(string name, string pass, DateTime login)
        {
            this.UserName = name;
            this.Password = pass;
            this.LastLogin = login;
        }
    }

    /// <summary>
    /// Users is a class that gives utils for accessing the user xml file
    /// Server only!
    /// </summary>
    public class Users
    {
        private string strPath;
        public ArrayList alUsers;
                    
        public Users()
        {
            this.strPath = Directory.GetCurrentDirectory();
            this.alUsers = this.LoadUsers();
        }
        
        public Users(string strPath)
        {
            this.strPath = strPath;
            this.alUsers = this.LoadUsers();
        }

        /// <summary>
        /// Check if User is available
        /// </summary>
        /// <param name="username">User Name</param>
        /// <returns></returns>
        public bool IsUser(string username)
        {
            if (alUsers == null)
                return false;

            foreach (User user in alUsers)
            {
                if (user.UserName == username)
                    return true;
            }

            return false;
        }

        /// <summary>
        /// Add new user to the existing users of ClockMark
        /// </summary>
        /// <param name="username">User Name</param>
        /// <param name="password">User Password in hash form</param>
        /// <returns></returns>
        public bool AddUser(string username, string password)
        {
            if (alUsers == null)
                return false;

            if (this.IsUser(username))
                return false;

            User newUser = new User(username, password, DateTime.Now);

            this.alUsers.Add(newUser);
            this.SaveSettings();

            return true;
        }

        /// <summary>
        /// Delete the existing user of ClockMark
        /// </summary>
        /// <param name="username">Existing User Name</param>
        /// <returns></returns>
        public bool DeleteUser(string username)
        {
            if (alUsers == null)
                return false;

            foreach (User user in alUsers)
            {
                if (user.UserName == username)
                {
                    alUsers.Remove(user);
                    return true;
                }
            }

            return false;
        }

        /// <summary>
        /// Check if the login user match the password
        /// this is the main method to be used from external solution
        /// </summary>
        /// <param name="username">User Name</param>
        /// <param name="password">User Password in hash form</param>
        /// <returns></returns>
        public bool IsLoginValid(string username, string password)
        {
            bool ret = false;

            if (alUsers == null)
                return ret;

            // look for the user in the arraylist and check if the password match
            foreach (User user in alUsers)
            {
                if (user.UserName == username)
                    if (user.Password == password)
                        ret = true;
            }

            return ret;
        }

        /// <summary>
        /// load all users from the xml file to the service. the method read the 
        /// xml file into XmlTextReader. then it searches for new xml elements.
        /// for each element the method extract its attributes
        /// </summary>
        public ArrayList LoadUsers()
        {
            XmlTextReader reader = null;
            string elementName = null;
            ArrayList al = new ArrayList();

            try
            {
                string fileFullName = strPath + "\\ClockMarkUsers.xml";
                
                if (File.Exists(fileFullName))
                {
                    reader = new XmlTextReader(fileFullName);
                    // define the Whitespace policy
                    reader.WhitespaceHandling = WhitespaceHandling.None;

                    while (reader.Read())
                    {
                        if (reader.NodeType == XmlNodeType.Element)
                        {
                            //for every new element there will be a new user inserted to the AL
                            elementName = reader.Name;
                            User user = new User();
                            user.UserName = elementName;
                            while (reader.MoveToNextAttribute())
                            {
                                if (reader.Name == "Password")
                                    user.Password = reader.Value;
                                else if (reader.Name == "LastLogin")
                                    user.LastLogin = DateTime.Parse(reader.Value);

                            }
                            //discard the Base element name "Users"
                            if (elementName != "Users")
                                al.Add(user);
                        }
                    }
                }
                else
                {
                    System.Diagnostics.Trace.WriteLine("File not Exist");
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
            }

            if (reader != null)
                reader.Close();

            return al;
        }

        /// <summary>
        /// SAVE all users to the xml file from the service. the method uses the 
        /// XmlTextWriter. It saves all the users to a new xml file. for each user 
        /// the method addes its attributes (xml attributes) to the user name (xml Element)
        /// </summary>
        public bool SaveSettings()
        {
            if (alUsers == null)
                return false;

            XmlTextWriter writer = new XmlTextWriter(strPath + "\\ClockMarkUsers.xml", null);
            writer.Formatting = Formatting.Indented;
            writer.Indentation = 4;
            writer.WriteComment("ClockMark users");
            writer.WriteStartElement("Users");

            foreach (User user in alUsers)
            {
                // The element name will be the user name
                // the attributes will contain the data to be saved for each user
                writer.WriteStartElement(user.UserName);
                writer.WriteAttributeString("Password", user.Password);
                writer.WriteAttributeString("LastLogin", user.LastLogin.ToString());
                writer.WriteEndElement();
            }

            writer.Flush();
            writer.Close();

            return true;
        }
    }
}
